'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _reducerTools = require('./reducer-tools');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function buildInitialState(_ref) {
  var fields = _ref.fields;
  var relations = _ref.relations;

  var state = {
    entities: []
  };

  _lodash2['default'].each(fields, function (__, indexName) {
    state[indexName] = {};
  });
  _lodash2['default'].each(relations, function (__, indexName) {
    state[indexName] = {};
  });

  return state;
}

function buildIndexBuilder(_ref2) {
  var fields = _ref2.fields;
  var relations = _ref2.relations;

  return function (entities) {
    var indexes = {};

    _lodash2['default'].each(fields, function (field, indexName) {
      indexes[indexName] = _reducerTools.buildIndex(entities, field);
    });

    _lodash2['default'].each(relations, function (relationField, indexName) {
      indexes[indexName] = _reducerTools.buildRelation(entities, relationField);
    });

    return indexes;
  };
}

exports['default'] = function (_ref3) {
  var ADD = _ref3.ADD;
  var REMOVE = _ref3.REMOVE;
  var MERGE = _ref3.MERGE;
  var indexMap = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var initialState = buildInitialState(indexMap);
  var indexBuilder = buildIndexBuilder(indexMap);

  return function (state, action) {
    if (state === undefined) state = initialState;

    var entities = undefined;
    var indexes = undefined;

    switch (action.type) {
      case ADD:
        entities = _reducerTools.addEntities(state.entities, action.entities);
        indexes = indexBuilder(entities);

        return _extends({}, state, { entities: entities }, indexes);

      case REMOVE:
        entities = _reducerTools.removeEntities(state.entities, action.ids);
        indexes = indexBuilder(entities);

        return _extends({}, state, { entities: entities }, indexes);

      case MERGE:
        var idsToReplace = action.entities.map(function (u) {
          return [u.id, u.__optimistic_id];
        });

        entities = _reducerTools.removeEntities(state.entities, idsToReplace);
        entities = _reducerTools.addEntities(entities, action.entities);
        indexes = indexBuilder(entities);

        return _extends({}, state, { entities: entities }, indexes);

      default:
        return state;
    }
  };
};

module.exports = exports['default'];